/**************************************************************************//**
*   @file   main.c
*   @brief  Implementation of main program for FMC-SDP evaluation projects.
*   @author ACozma (andrei.cozma@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: $WCREV$
******************************************************************************/

/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include <stdio.h>
#include "platform.h"
#include "xil_io.h"
#include "xparameters.h"
#include "dma.h"

/*****************************************************************************/
/************************ Constants Definitions ******************************/
/*****************************************************************************/
#define SAMPLE_NO 8*1024

/*****************************************************************************/
/************************ Variables Definitions ******************************/
/*****************************************************************************/
u32 buffer[256];							//buffer to store the SG DMA descriptors
u32* pBuffer = buffer;						//pointer to the descriptors buffer
stTransferDescriptor* pTransferDescriptor1;	//SG DMA descriptor 1
stTransferDescriptor* pTransferDescriptor2;	//SG DMA descriptor 2
volatile u32 startAcquisition;				//starts one acquisition cycle
volatile u32* pStartAcquisition = &startAcquisition; //pointer to the startAcquisition variable
u32 rxData[SAMPLE_NO];						//array to store the data read from the ADC
u32* pRxData = rxData; 						//pointer to the RX buffer

/**************************************************************************//**
* @brief Reads data from an ADC using SG DMA.
*
* @param buf - Buffer where to store the read data
* @param size - Size in bytes of the data to read
* @return None
******************************************************************************/
void ReadAdcData(u32* buf, u32 size)
{
	pTransferDescriptor1->NXTDESC_PNTR 	= (u32)pTransferDescriptor2;
	pTransferDescriptor1->BUFFER_ADDR   = (u32)buf;
	pTransferDescriptor1->CONTROL 	   	= (u32)size/2;
	pTransferDescriptor1->STATUS 	    = (u32)0;

	pTransferDescriptor2->NXTDESC_PNTR 	= (u32)pTransferDescriptor1;
	pTransferDescriptor2->BUFFER_ADDR   = (u32)buf + size/2;
	pTransferDescriptor2->CONTROL 	   	= (u32)size/2;
	pTransferDescriptor2->STATUS 	    = (u32)0;

	DMA_TransferDataRX(pTransferDescriptor1, pTransferDescriptor2);
	while(!DMA_IsTransferCompleteRX());
}

/**************************************************************************//**
* @brief Main function implementation.
*
* @return None
******************************************************************************/
int main()
{
	init_platform();

	//allocate the SG DMA descriptors (the descriptors must be 64 bytes aligned)
	while((u32)pBuffer % 0x40)
	{
		pBuffer++;
	}
	pTransferDescriptor1 = (stTransferDescriptor*)pBuffer;
	pTransferDescriptor2 = (stTransferDescriptor*)((char*)pTransferDescriptor1 + 0x40);

	xil_printf("\n-------------------------------------\n\r");
	xil_printf("Waiting for command from the PC...\n\r");
	while(!startAcquisition);
	xil_printf("Starting a new data acquisition...\n\r");

	xil_printf("Reading data from the ADC...\n\r");
	//read a first batch of data to clear the ADC buffer
	ReadAdcData(rxData, (SAMPLE_NO * sizeof(u32)) / 2);

	//read the actual data from the ADC
	ReadAdcData(rxData, (SAMPLE_NO * sizeof(u32)) / 2);
	ReadAdcData(rxData+SAMPLE_NO/2, (SAMPLE_NO * sizeof(u32)) / 2);

	while(startAcquisition);
	xil_printf("Data read complete. Data was stored in the file Acquisition.csv.\n\r");

    cleanup_platform();

    return 0;
}
